# -*- coding: utf-8 -*-
"""
templatehooks.templatetags.templatehooks
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Provides easy access to templatehooks via a templatetag.

:copyright: 2010, Pascal Hartig <phartig@rdrei.net>
:license: commercial
"""
# pylint: disable-msg=C0103,R0903

from django.dispatch.dispatcher import Signal
from ..registry import registry
from django import template

register = template.Library()


class TemplateHookNode(template.Node):
    """
    Renders the {% hook %} template tag.
    """

    def __init__(self, name, as_var=None):
        self.name = name
        self.as_var = as_var

    def render(self, context):
        """
        Fires the given signal and renders the list values in response.
        """

        content = registry.get_content(self.name, context)

        if self.as_var:
            context[self.as_var] = rendered_content
            return ''

        return rendered_content


# pylint: disable-msg=W0613
@register.tag(name="hook")
def do_template_hook(parser, token):
    """
    Extends your templates easily with pluggable apps.

    Example use::
        {% hook {signal_name} %}
        or
        {% hook {signal_name} as {context_var} %}

    The signal must be defined, currently inside this module's models.

    To use a template hook you connect to the signal and modify the signal's
    content variable::

        from templatetags.models import my_hook_name

        def extend_hook(signal, **kwargs):
            kwargs['content'].append("Hello World!")

        my_hook_name.connect(extend_hook)
    """

    bits = token.split_contents()

    as_var = None
    if len(bits) == 2:
        name = bits[1]
    elif len(bits) == 4:
        name = bits[1]
        as_var = bits[3]
    else:
        raise template.TemplateSyntaxErrors("{0} takes either two or four "
            "arguments: {% hook {signal_name} [as {context_var}] %}")

    return TemplateHookNode(name, as_var)



"""
=====================
Django Template Hooks
=====================

Provides template hooks to provide an easy interface for pluggable apps to
extend your templates.
"""

try:
    from setuptools.core import setup
except ImportError:
    from distutils.core import setup

setup(
    name='django-templatehooks',
    version='0.1.0',
    author='Pascal Hartig',
    author_email='phartig@weluse.de',
    packages=['templatehooks', 'templatehooks.templatetags'],
    url='http://github.com/weluse/django-templatetags',
    license='MIT',
    description='Template hooks for django',
    long_description=__doc__,
    zip_safe=False
    # test_suite='still to be written'
)



# -*- coding: utf-8 -*-
"""
templatehooks.registry
~~~~~~~~~~~~~~~~~~~~~~

Registry for template hook signals.

:copyright: 2010, weluse (http://weluse.de)
:author: Pascal Hartig <phartig@weluse.de>
:license: 2010
"""

from django.dispatch.dispatcher import Signal


class HookRegistry(object):
    """Storage for template hooks."""

    def __init__(self):
        self._registry = {}

    def register(self, name, signal=None):
        """
        Register a new template hook or override an existing.

        :param name: Name of the template hook.
        :param signal: The signal to use for the hook. If not provided, it
        is created.
        """

        if signal:
            if not isinstance(signal, Signal):
                raise TypeError("Signal {0} is not of type "
                    "django.dispatch.dispatcher.Signal!".format(name))
            if 'content' not in signal.providing_args:
                raise TypeError("Signal {0} does not provide a 'content' "
                    "parameter!".format(name))
        else:
            signal = Signal(providing_args=['content'])

        self._registry[name] = signal

    def connect(self, name, fn):
        """
        Connect a function ``fn`` to the template hook ``name``.

        An example hook could look like this::
            function my_hook(sender, **kwargs):
                # Get the request from context
                request = sender['request']
                kwargs['content'].append("Hello, {0}!".format(request.user))
        """

        signal = self._registry[name]
        signal.connect(fn)

    def get_content(self, name, context):
        """
        Get the content of a template hook. Used internally by the hook
        templatetag.
        """

        signal = self._registry[name]
        content = []
        signal.send(sender=context, content=content)

        return '\n'.join(content)


registry = HookRegistry()
__all__ = ('registry',)
